/*
	'3D No More' kwin client

	Copyright (C) 2002 Yuheng Xie <elephant@linux.net.cn>

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.
*/

#include "k3dnomore.h"

using namespace KWinInternal;

extern "C"
{
	Client * allocate(Workspace *ws, WId w, int tool)
	{
		return new K3DNoMore(ws, w, tool != 0);
	}
	void init() {}
	void reset()
	{
		Workspace::self()->slotResetAllClientsDelayed();
	}
	void deinit() {}
}

K3DNoMoreButton::K3DNoMoreButton(QWidget *parent, const QString &tip, K3DNoMoreButtonType t)
	:KWinButton(parent, 0, tip), buttonType(t), status(0), mouseOver(false)
{
	KConfig conf("kwin3dnomorerc");
	conf.setGroup("General");
	colorGears = conf.readColorEntry("GearsColor", &QColor(168, 204, 240));
	colorGearsDis = conf.readColorEntry("GearsColorDis", &QColor(220, 220, 220));
	colorButtons = conf.readColorEntry("ButtonsColor", &QColor(90, 159, 228));
	mincolorButtons = conf.readColorEntry("minButtonsColor", &QColor(90, 159, 228));
	maxcolorButtons = conf.readColorEntry("maxButtonsColor", &QColor(90, 159, 228));
	closecolorButtons = conf.readColorEntry("closeButtonsColor", &QColor(90, 159, 228));
	colorButtonsDis = conf.readColorEntry("ButtonsColorDis", &QColor(188, 188, 188));

	setBackgroundMode(NoBackground);
}

void K3DNoMoreButton::drawButton(QPainter *p)
{
	Client *client = (Client *) parentWidget();
	if (buttonType == _3Menu)
	{
		p->fillRect(rect(), options->color(Options::TitleBar, client->isActive()));
		QPixmap icon = ((Client *)parentWidget())->miniIcon();
		int x0 = (18 - icon.width()) / 2;
		int y0 = (18 - icon.height()) / 2;
		if (!icon.isNull()) p->drawPixmap(x0, y0, icon);
	}
	else if (buttonType == _3Sticky)
	{
		char *buttonData;
		if (!status) buttonData = (char *)
			"000000000000000000"
			"000000000000000000"
			"000000000000000000"
			"000000000002210000"
			"000000000012221000"
			"000000000122222000"
			"000000001222222000"
			"000002112222210000"
			"000001222222100000"
			"000000122221000000"
			"000000122210000000"
			"000001211210000000"
			"000012100120000000"
			"000121000000000000"
			"000210000000000000"
			"000000000000000000"
			"000000000000000000"
			"000000000000000000";
		else buttonData = (char *)
			"000000000000000000"
			"000000000000000000"
			"000000000000000000"
			"000000000000000000"
			"000000000000000000"
			"000000012210000000"
			"000000221122000000"
			"000001212212100000"
			"000002122221200000"
			"000002122221200000"
			"000001212212100000"
			"000000221122000000"
			"000000012210000000"
			"000000000000000000"
			"000000000000000000"
			"000000000000000000"
			"000000000000000000"
			"000000000000000000";
		int i, j;
		QColor cf = options->color(Options::Font, client->isActive());
		QColor c = options->color(Options::TitleBar, client->isActive());
		p->fillRect(rect(), c);
		p->setPen(cf);
		for (i = 3; i < 15; i ++)
			for (j = 3; j < 15; j ++)
				if (buttonData[i + j * 18] == '2') p->drawPoint(i, j);
		p->setPen(QColor(
			(cf.red() + c.red()) / 2, (cf.green() + c.green()) / 2, (cf.blue() + c.blue()) / 2));
		for (i = 3; i < 15; i ++)
			for (j = 3; j < 15; j ++)
				if (buttonData[i + j * 18] == '1') p->drawPoint(i, j);
	}
	else
	{
		char *buttonData = 0;
		if (buttonType == _3Help) buttonData = (char *)
			"000000000000000000"
			"000000000000000000"
			"000122222222221000"
			"001210000000012100"
			"002100122221001200"
			"002001222222100200"
			"002002210012200200"
			"002000000012200200"
			"002000000122100200"
			"002000001221000200"
			"002000001221000200"
			"002000000000000200"
			"002000001221000200"
			"002100001221001200"
			"001210000000012100"
			"000122222222221000"
			"000000000000000000"
			"000000000000000000";
		if (buttonType == _3Minimize) buttonData = (char *)
			"000000000000000000"
			"000000000000000000"
			"000122222222221000"
			"001210000000012100"
			"002100000000001200"
			"002000000000000200"
			"002000000000000200"
			"002000000000000200"
			"002000000000000200"
			"002000000000000200"
			"002000000000000200"
			"002001222222100200"
			"002001222222100200"
			"002100000000001200"
			"001210000000012100"
			"000122222222221000"
			"000000000000000000"
			"000000000000000000";
		if ((buttonType == _3Maximize) && !status) buttonData = (char *)
			"000000000000000000"
			"000000000000000000"
			"000122222222221000"
			"001210000000012100"
			"002100000000001200"
			"002001222222100200"
			"002002222222200200"
			"002002000000200200"
			"002002000000200200"
			"002002000000200200"
			"002002000000200200"
			"002002000000200200"
			"002001222222100200"
			"002100000000001200"
			"001210000000012100"
			"000122222222221000"
			"000000000000000000"
			"000000000000000000";
		if ((buttonType == _3Maximize) && status) buttonData = (char *)
			"000000000000000000"
			"000000000000000000"
			"000122222222221000"
			"001210000000012100"
			"002100000000001200"
			"002000012222100200"
			"002000020000200200"
			"002001222210200200"
			"002002222220200200"
			"002002000020200200"
			"002002000022100200"
			"002002000020000200"
			"002001222210000200"
			"002100000000001200"
			"001210000000012100"
			"000122222222221000"
			"000000000000000000"
			"000000000000000000";
		if (buttonType == _3Close) buttonData = (char *)
			"000000000000000000"
			"000000000000000000"
			"000122222222221000"
			"001210000000012100"
			"002100000000001200"
			"002002100001200200"
			"002001210012100200"
			"002000121121000200"
			"002000012210000200"
			"002000012210000200"
			"002000121121000200"
			"002001210012100200"
			"002002100001200200"
			"002100000000001200"
			"001210000000012100"
			"000122222222221000"
			"000000000000000000"
			"000000000000000000";
		QColor c = client->isActive() ? colorGears : colorGearsDis;
		p->fillRect(rect(), c);
		if (buttonData)
		{
			QColor curcolorButtons = colorButtons;
			QColor cc = c;

			if (buttonType == _3Help) curcolorButtons = colorButtons;
			if (buttonType == _3Minimize) curcolorButtons = mincolorButtons;
			if (buttonType == _3Maximize) curcolorButtons = maxcolorButtons;
			if (buttonType == _3Close) curcolorButtons = closecolorButtons;

			if (!mouseOver) cc = client->isActive() ? curcolorButtons : colorButtonsDis;
			p->fillRect(QRect(3, 3, 12, 12), cc);

			int i, j;
			p->setPen(Qt::white);
			for (i = 2; i < 16; i ++)
				for (j = 2; j < 16; j ++)
					if (buttonData[i + j * 18] == '2') p->drawPoint(i, j);
			p->setPen(QColor((c.red() + 255) / 2, (c.green() + 255) / 2, (c.blue() + 255) / 2));
			for (i = 2; i < 16; i ++)
				for (j = 2; j < 16; j ++)
				{
					if ((i >= 3) && (i < 15) && (j >= 3) && (j < 15)) continue;
					if (buttonData[i + j * 18] == '1') p->drawPoint(i, j);
				}
			p->setPen(QColor((cc.red() + 255) / 2, (cc.green() + 255) / 2, (cc.blue() + 255) / 2));
			for (i = 3; i < 15; i ++)
				for (j = 3; j < 15; j ++)
					if (buttonData[i + j * 18] == '1') p->drawPoint(i, j);
		}
	}
}

void K3DNoMoreButton::mousePressEvent(QMouseEvent *e)
{
	QMouseEvent me(e->type(), e->pos(), e->globalPos(), LeftButton, e->state());
	KWinButton::mousePressEvent(&me);
}

void K3DNoMoreButton::mouseReleaseEvent(QMouseEvent *e)
{
	if (buttonType == _3Maximize) emit(clicked(e->button()));
	QMouseEvent me(e->type(), e->pos(), e->globalPos(), LeftButton, e->state());
	KWinButton::mouseReleaseEvent(&me);
}

void K3DNoMoreButton::enterEvent(QEvent *e)
{
	mouseOver = true;
	repaint(false);
	KWinButton::enterEvent(e);
}

void K3DNoMoreButton::leaveEvent(QEvent *e)
{
	mouseOver = false;
	repaint(false);
	KWinButton::leaveEvent(e);
}

void K3DNoMoreButton::slotStatusChange(bool s)
{
	if ((buttonType == _3Maximize) || (buttonType == _3Sticky))
	{
		status = s;
		repaint(false);
	}
}

K3DNoMore::K3DNoMore(Workspace *ws, WId w, bool tool, QWidget *parent, const char *name)
	:Client(ws, w, parent, name, WResizeNoErase)
{
	KConfig conf("kwin3dnomorerc");
	conf.setGroup("General");
	use3DFrame = conf.readBoolEntry("Use3DFrame", false);
	invertGears = conf.readBoolEntry("InvertGears", false);
	colorDashedLines = conf.readColorEntry("DashedLinesColor", &QColor(40, 70, 100));
	colorDashedLinesDis = conf.readColorEntry("DashedLinesColorDis", &QColor(100, 100, 100));
	colorGears = conf.readColorEntry("GearsColor", &QColor(168, 204, 240));
	colorGearsDis = conf.readColorEntry("GearsColorDis", &QColor(220, 220, 220));

	TITLE_HEIGHT = 18;
	BORDER_SIZE = use3DFrame ? 3 : 3;
	CORNER_SIZE = TITLE_HEIGHT + BORDER_SIZE;
	
	setBackgroundMode(NoBackground);
	buttons[0] = new K3DNoMoreButton(this, i18n("Menu"), _3Menu);
	buttons[1] = new K3DNoMoreButton(this, i18n("Sticky"), _3Sticky);
	if (providesContextHelp()) buttons[2] = new K3DNoMoreButton(this, i18n("Help"), _3Help);
	else buttons[2] = 0;
	buttons[3] = new K3DNoMoreButton(this, i18n("Minimize"), _3Minimize);
	buttons[4] = new K3DNoMoreButton(this, i18n("Maximize"), _3Maximize);
	buttons[5] = new K3DNoMoreButton(this, i18n("Close"), _3Close);
	for (int i = 0; i < 6; i ++)
	{
		if (!buttons[i]) continue;
		buttons[i]->setMouseTracking(false);
		buttons[i]->setFocusPolicy(NoFocus);
		buttons[i]->setFixedSize(18, 18);
	}
	connect(buttons[0], SIGNAL(pressed()), this, SLOT(slotMenu()));
	connect(buttons[1], SIGNAL(clicked()), this, SLOT(toggleSticky()));
	if (providesContextHelp()) connect(buttons[2], SIGNAL(clicked()), this, SLOT(contextHelp()));
	connect(buttons[3], SIGNAL(clicked()), this, SLOT(iconify()));
	connect(buttons[4], SIGNAL(clicked(int)), this, SLOT(slotMaximize(int)));
	connect(this, SIGNAL(maximizeChange(bool)), buttons[4], SLOT(slotStatusChange(bool)));
	connect(buttons[5], SIGNAL(clicked()), this, SLOT(closeWindow()));
	if (isTransient()) buttons[4]->hide();
	resetLayout();
}

void K3DNoMore::paintEvent(QPaintEvent *e)
{
	QPainter p(this);
	QRect titleRect = titleSpacer->geometry();

	p.setClipRegion(e->region() - titleRect);

	short int titleDataMap[128];
	titleDataMap['.'] = 0xFF; titleDataMap['+'] = 0xF6; titleDataMap['@'] = 0xE5;
	titleDataMap['#'] = 0xCA; titleDataMap['$'] = 0x95; titleDataMap['%'] = 0x61;
	titleDataMap['&'] = 0x2C; titleDataMap['*'] = 0x09; titleDataMap['='] = 0x00;
	titleDataMap['-'] = 0xA7; titleDataMap[';'] = 0xC1; titleDataMap['>'] = 0x58;
	titleDataMap[','] = 0x4F; titleDataMap['\''] = 0xD3; titleDataMap[')'] = 0x9E;
	titleDataMap['!'] = 0x8D; titleDataMap['~'] = 0x84; titleDataMap['{'] = 0x7B;
	titleDataMap[']'] = 0xB0; titleDataMap['^'] = 0x23; titleDataMap['/'] = 0x1A;
	titleDataMap['('] = 0x12; titleDataMap['_'] = 0xED; titleDataMap[':'] = 0x6A;
	titleDataMap['<'] = 0x35; titleDataMap['['] = 0x72; titleDataMap['}'] = 0x3E;
	titleDataMap['|'] = 0xDC; titleDataMap['1'] = 0xB9; titleDataMap['2'] = 0xFA;
	titleDataMap['3'] = 0x46; titleDataMap['4'] = 0xFE; titleDataMap['5'] = 0xFC;
	titleDataMap['6'] = 0xF9; titleDataMap['7'] = 0xFD; titleDataMap['8'] = 0xFB;
	titleDataMap['9'] = 0xF8;

	char titleData[] =
		"...++@#$%&*============================================================-@============;+>=====,+#======-@================"
		"........................................................................;===========,.'-)$!!~{].>====={.,&^^/((*========"
		"...++_'):<(============================================================@+!^=========-_*=======*_-=====>.[==============="
		"..........................................................................+]}======='@-))$!~~{[|#=====('.@{^/(**========"
		"...++_|;!:}/(*==============================================&+.++$=======^!_.;======|1=========##=======>'.@%==========="
		".............................................................'(*%........++_+.}=====1+-)$$!~{{{+]=========,@_(**========"
		"...2++_@#]![>}&^/(****======================================#~==(_,=====}*==^.{=====:.>=======:.>=========='#==========="
		".............................................................}===-........__@.-=====*@+]$!!~{$+'==========*+-(*========="
		"....2+++_@|#1-$~[:%>,3<^/**=========================[+|@:==[@====,+>*:_1<''&#+}======<__~<^&!_@^======},%[!.:==========="
		"....................................................+>*[_..+[====*!_._:==(@..%========&1.....1^======1....._}**========="
		"....4.56+++___@|''#;1]$~:>}^(*=====================1#===/)-&=======(%^===&+.{===========^%::^=======>.!3&/*============="
		".....................................................%===================1.$=======================*@|}<&^//(**========="
		".........78++++++++___@|#1${>&(*====================[@/=================3._*=======================*_]=================="
		"......................................................1=================$..!===(%])^================-+3&&^//(*=========="
		"................569+++++__|;$[}(*==*==**==***========>+*=====}-||$&=====}+#.:[;+._++~*==============3.%================="
		"......................................................;=====!.....+>=====#....+@';#@.:======^>{3=====@#&^^/((*=========="
		".....................+++++_@;$:<^(*/(*^/(*^^/(*==*3$@+>====[_}===={+&====/{]@.-====(.{====*'...+:====!+^================"
		"..................................................+1:/====*@........]========:+#;;11.!===={.;${'.[==*).{^//(**==========";

	QRect t0 = QRect(0, 0, titleRect.width(), titleRect.height());
	QPixmap buffer(t0.width(), t0.height());
	QPainter p2(&buffer);
	QColor ctitlebar = options->color(Options::TitleBar, isActive());
	QColor ctitleblend = options->color(Options::TitleBlend, isActive());
	QColor cgears = isActive() ? colorGears : colorGearsDis;
	KPixmap grad;
	grad.resize(t0.width() - 120, t0.height());
	KPixmapEffect::gradient(grad, ctitlebar, ctitleblend, KPixmapEffect::HorizontalGradient);
	p2.drawPixmap(0, 0, grad);
	int lr = ctitleblend.red(), lg = ctitleblend.green(), lb = ctitleblend.blue();
	int rr = cgears.red(), rg = cgears.green(), rb = cgears.blue();
	int i, j;
	for (i = 0; i < 120; i ++)
		for (j = 0; j < 18; j ++)
		{
			int r;
			if (!invertGears) r = titleDataMap[titleData[i + j * 120]];
			else r = 255 - titleDataMap[titleData[(119 - i) + (17 - j) * 120]];
			p2.setPen(QColor(
				(lr * r + rr * (255 - r)) / 255,
				(lg * r + rg * (255 - r)) / 255,
				(lb * r + rb * (255 - r)) / 255
			));
			p2.drawPoint(t0.width() - 120 + i, j);
		}
	p2.setFont(options->font(isActive()));
	p2.setPen(options->color(Options::Font, isActive()));
	t0.setLeft(3);
	p2.drawText(t0, AlignLeft | AlignVCenter | SingleLine, caption());
	p2.end();

	QColor cframe = options->color(Options::Frame, isActive());
	p.fillRect(rect(), cframe);
	int w = width(), h = height();
	if (!use3DFrame)
	{
		int cr = cframe.red(), cg = cframe.green(), cb = cframe.blue();

		QPixmap srdTop = QPixmap::grabWindow(qt_xrootwin(), x(), y() - 1, w, 1);
		if (y() - 1 < 0) srdTop.fill(Qt::black);
		QImage img = srdTop.convertToImage();
		if (img.depth() != 32) img = img.convertDepth(32);
		QRgb *bits = (QRgb *)img.bits();
		for (i = 0; i < w; i ++)
			bits[i] = qRgb(
				(qRed(bits[i]) + cr) / 2,
				(qGreen(bits[i]) + cg) / 2,
				(qBlue(bits[i]) + cb) / 2
			);
		srdTop.convertFromImage(img);
		p.drawPixmap(0, 0, srdTop);

		QPixmap srdBottom = QPixmap::grabWindow(qt_xrootwin(), x(), y() + h, w, 1);
		if (y() + h >= QApplication::desktop()->height()) srdBottom.fill(Qt::black);
		img = srdBottom.convertToImage();
		if (img.depth() != 32) img = img.convertDepth(32);
		bits = (QRgb *)img.bits();
		for (i = 0; i < w; i ++)
			bits[i] = qRgb(
				(qRed(bits[i]) + cr) / 2,
				(qGreen(bits[i]) + cg) / 2,
				(qBlue(bits[i]) + cb) / 2
			);
		srdBottom.convertFromImage(img);
		p.drawPixmap(0, h - 1, srdBottom);

		QPixmap srdLeft = QPixmap::grabWindow(qt_xrootwin(), x() - 1, y(), 1, h);
		if (x() - 1 < 0) srdLeft.fill(Qt::black);
		img = srdLeft.convertToImage();
		if (img.depth() != 32) img = img.convertDepth(32);
		bits = (QRgb *)img.bits();
		for (i = 0; i < h; i ++)
			bits[i] = qRgb(
				(qRed(bits[i]) + cr) / 2,
				(qGreen(bits[i]) + cg) / 2,
				(qBlue(bits[i]) + cb) / 2
			);
		srdLeft.convertFromImage(img);
		p.drawPixmap(0, 0, srdLeft);

		QPixmap srdRight = QPixmap::grabWindow(qt_xrootwin(), x() + w, y(), 1, h);
		if (x() + w >= QApplication::desktop()->width()) srdRight.fill(Qt::black);
		img = srdRight.convertToImage();
		if (img.depth() != 32) img = img.convertDepth(32);
		bits = (QRgb *)img.bits();
		for (i = 0; i < h; i ++)
			bits[i] = qRgb(
				(qRed(bits[i]) + cr) / 2,
				(qGreen(bits[i]) + cg) / 2,
				(qBlue(bits[i]) + cb) / 2
			);
		srdRight.convertFromImage(img);
		p.drawPixmap(w - 1, 0, srdRight);

		p.setPen(isActive() ? colorDashedLines : colorDashedLinesDis);
		for (i = 1; i <= w - 2; i += 6)
		{
			p.drawLine(i, 1, (i + 3 <= w - 2) ? i + 3 : w - 2, 1);
			p.drawLine(i, h - 2, (i + 3 <= w - 2) ? i + 3 : w - 2, h - 2);
		}
		for (j = 1; j <= h - 2; j += 6)
		{
			p.drawLine(1, j, 1, (j + 3 <= h - 2) ? j + 3 : h - 2);
			p.drawLine(w - 2, j, w - 2, (j + 3 <= h - 2) ? j + 3 : h - 2);
		}
	}
	else
	{
		p.setPen(options->color(Options::Frame, isActive()).light());
		p.drawLine(1, 1, 1, h - 3);
		p.setPen(options->color(Options::Frame, isActive()).dark(125));
		p.drawLine(w - 2, 2, w - 2, h - 2);
		p.setPen(options->color(Options::Frame, isActive()).dark(200));
		p.drawLine(w - 1, 1, w - 1, h - 1);
		p.setPen(options->color(Options::Frame, isActive()).light());
		p.drawLine(1, 1, w - 3, 1);
		p.setPen(options->color(Options::Frame, isActive()).dark(125));
		p.drawLine(2, h - 2, w - 2, h - 2);
		p.setPen(options->color(Options::Frame, isActive()).dark(200));
		p.drawLine(1, h - 1, w - 1, h - 1);
	}
	p.end();

	bitBlt(this, titleRect.topLeft(), &buffer);

	for (i = 0; i < 6; i ++) if (buttons[i]) buttons[i]->repaint(false);
}

Client::MousePosition K3DNoMore::mousePosition(const QPoint &pos) const
{
	int x = pos.x();
	int y = pos.y();

	if (y < BORDER_SIZE)
	{
		if (x < CORNER_SIZE) return Client::TopLeft;
		else if (x < width() - CORNER_SIZE) return Client::Top;
		else return Client::TopRight;
	}
	else if (y < CORNER_SIZE)
	{
		if (x < BORDER_SIZE) return Client::TopLeft;
		else if (x < width() - BORDER_SIZE) return Client::Center;
		else return Client::TopRight;
	}
	else if (y < height() - CORNER_SIZE)
	{
		if (x < BORDER_SIZE) return Client::Left;
		else if (x < width() - BORDER_SIZE) return Client::Center;
		else return Client::Right;
	}
	else
	{
		if (x < CORNER_SIZE) return Client::BottomLeft;
		else if (x < width() - CORNER_SIZE) return Client::Bottom;
		else return Client::BottomRight;
	}

	return Client::mousePosition(pos);
}

void K3DNoMore::mouseDoubleClickEvent(QMouseEvent *e)
{
	if (titleSpacer->geometry().contains(e->pos()))
		workspace()->performWindowOperation(this, options->operationTitlebarDblClick());
}

void K3DNoMore::moveEvent(QMoveEvent *e)
{
	if (use3DFrame) return;

	QPainter p(this);
	int w = width(), h = height(), i;
	QColor cframe = options->color(Options::Frame, isActive());
	int cr = cframe.red(), cg = cframe.green(), cb = cframe.blue();

	QPixmap srdTop = QPixmap::grabWindow(qt_xrootwin(), x(), y() - 1, w, 1);
	if (y() - 1 < 0) srdTop.fill(Qt::black);
	QImage img = srdTop.convertToImage();
	if (img.depth() != 32) img = img.convertDepth(32);
	QRgb *bits = (QRgb *)img.bits();
	for (i = 0; i < w; i ++)
		bits[i] = qRgb(
			(qRed(bits[i]) + cr) / 2,
			(qGreen(bits[i]) + cg) / 2,
			(qBlue(bits[i]) + cb) / 2
		);
	srdTop.convertFromImage(img);
	p.drawPixmap(0, 0, srdTop);

	QPixmap srdBottom = QPixmap::grabWindow(qt_xrootwin(), x(), y() + h, w, 1);
	if (y() + h >= QApplication::desktop()->height()) srdBottom.fill(Qt::black);
	img = srdBottom.convertToImage();
	if (img.depth() != 32) img = img.convertDepth(32);
	bits = (QRgb *)img.bits();
	for (i = 0; i < w; i ++)
		bits[i] = qRgb(
			(qRed(bits[i]) + cr) / 2,
			(qGreen(bits[i]) + cg) / 2,
			(qBlue(bits[i]) + cb) / 2
		);
	srdBottom.convertFromImage(img);
	p.drawPixmap(0, h - 1, srdBottom);

	QPixmap srdLeft = QPixmap::grabWindow(qt_xrootwin(), x() - 1, y(), 1, h);
	if (x() - 1 < 0) srdLeft.fill(Qt::black);
	img = srdLeft.convertToImage();
	if (img.depth() != 32) img = img.convertDepth(32);
	bits = (QRgb *)img.bits();
	for (i = 0; i < h; i ++)
		bits[i] = qRgb(
			(qRed(bits[i]) + cr) / 2,
			(qGreen(bits[i]) + cg) / 2,
			(qBlue(bits[i]) + cb) / 2
		);
	srdLeft.convertFromImage(img);
	p.drawPixmap(0, 0, srdLeft);

	QPixmap srdRight = QPixmap::grabWindow(qt_xrootwin(), x() + w, y(), 1, h);
	if (x() + w >= QApplication::desktop()->width()) srdRight.fill(Qt::black);
	img = srdRight.convertToImage();
	if (img.depth() != 32) img = img.convertDepth(32);
	bits = (QRgb *)img.bits();
	for (i = 0; i < h; i ++)
		bits[i] = qRgb(
			(qRed(bits[i]) + cr) / 2,
			(qGreen(bits[i]) + cg) / 2,
			(qBlue(bits[i]) + cb) / 2
		);
	srdRight.convertFromImage(img);
	p.drawPixmap(w - 1, 0, srdRight);
	p.end();
}

void K3DNoMore::stickyChange(bool s)
{
	buttons[1]->slotStatusChange(s);
}

void K3DNoMore::slotMenu()
{
	QPoint menuPoint(
		buttons[0]->rect().bottomLeft().x(), buttons[0]->rect().bottomLeft().y() + 1);
	workspace()->clientPopup(this)->popup(buttons[0]->mapToGlobal(menuPoint));
}

void K3DNoMore::slotMaximize(int b)
{
	if (b == MidButton) maximize(MaximizeVertical);
	else if (b == RightButton) maximize(MaximizeHorizontal);
	else maximize(MaximizeFull);
}

void K3DNoMore::resetLayout()
{
	mainLayout = new QVBoxLayout(this, BORDER_SIZE, 0);
	QBoxLayout *topLayout = new QHBoxLayout(mainLayout, 0, 0);
	titleSpacer = new QSpacerItem(0, TITLE_HEIGHT, QSizePolicy::Expanding, QSizePolicy::Fixed);
	topLayout->addWidget(buttons[0]);
	topLayout->addWidget(buttons[1]);
	topLayout->addItem(titleSpacer);
	if (providesContextHelp()) topLayout->addWidget(buttons[2]);
	topLayout->addWidget(buttons[3]);
	topLayout->addWidget(buttons[4]);
	topLayout->addWidget(buttons[5]);
	mainLayout->addWidget(windowWrapper());
}

// vim: ts=2
