/*
  '3D No More' kwin client

  Copyright (C) 2002 Yuheng Xie <elephant@linux.net.cn>

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; see the file COPYING.  If not, write to
  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  Boston, MA 02111-1307, USA.
*/

#include <qimage.h>
#include <qpainter.h>
#include <qlayout.h>
#include <qdesktopwidget.h>
#include <kwin/workspace.h>
#include <kwin/client.h>
#include <kwin/kwinbutton.h>
#include <kpixmap.h>
#include <kpixmapeffect.h>
#include <kconfig.h>
#include <klocale.h>

namespace KWinInternal {

enum K3DNoMoreButtonType { _3Menu, _3Sticky, _3Help, _3Minimize, _3Maximize, _3Close };

class K3DNoMoreButton : public KWinInternal::KWinButton
{
	Q_OBJECT

public:
	K3DNoMoreButton(QWidget *parent, const QString &tip, K3DNoMoreButtonType t);
	virtual void drawButton(QPainter *p);
	virtual void mousePressEvent(QMouseEvent *e);
	virtual void mouseReleaseEvent(QMouseEvent *e);
	virtual void enterEvent(QEvent *e);
	virtual void leaveEvent(QEvent *e);
signals:
	void clicked(int b);
public slots:
	void slotStatusChange(bool s);
private:
	K3DNoMoreButtonType buttonType;
	int status;
	bool mouseOver;
	QColor colorGears, colorGearsDis, colorButtons, colorButtonsDis;
	QColor mincolorButtons, maxcolorButtons, closecolorButtons;
};

class K3DNoMore : public KWinInternal::Client
{
	Q_OBJECT

public:
	K3DNoMore(Workspace *ws, WId w, bool tool, QWidget *parent = 0, const char *name = 0);
protected:
	virtual void paintEvent(QPaintEvent *e);
	virtual Client::MousePosition mousePosition(const QPoint &pos) const;
	virtual void mouseDoubleClickEvent(QMouseEvent *e);
	virtual void moveEvent(QMoveEvent *e);
	virtual void stickyChange(bool s);
signals:
	void maximizeChange(bool m);
public slots:
	void slotMenu();
	void slotMaximize(int b);
private:
	void resetLayout();

	int TITLE_HEIGHT;
	int BORDER_SIZE;
	int CORNER_SIZE;

	QBoxLayout *mainLayout;
	QSpacerItem *titleSpacer;
	K3DNoMoreButton *buttons[6];
	bool use3DFrame, invertGears;
	QColor colorDashedLines, colorDashedLinesDis, colorGears, colorGearsDis;
};

} // end namespace KWinInternal

// vim: ts=2
